/* FILE: sysdep.c             Copyright (c), Symmetric Research, 2000
 *
 * This file provides wrappers for the system calls which depend on the
 * Linux kernel version.  These are used by the kernel mode device driver 
 * code.  The version dependent defines in the top part of this file are 
 * the same for all Symmetric Research products.
 *
 * The bottom half of this file has been customized for the DSP32C.
 *
 */

#include <linux/autoconf.h>           /* retrieve the CONFIG_* macros */
#if defined(CONFIG_MODVERSIONS) && !defined(MODVERSIONS)
#   define MODVERSIONS                /* force it on */
#endif
#ifdef MODVERSIONS
#  include <linux/modversions.h>
#endif


#include <linux/kernel.h>
#include <linux/module.h>     /* MOD_INC_USE_COUNT, MODULE_PARM        */
#include <linux/ioport.h>     /* io port region checking and reserving */
#include <linux/sched.h>      /* irq and task queue info               */
#include <linux/timer.h>      /* timer info                            */
#include <linux/fs.h>         /* file structures, register_chrdev      */


#include "sysdep.h"



/* 
 * This code is based on the file sysdep-2.1.h that comes with the code
 * samples accompanying the O'Reilly book "Linux Device Drivers" by
 * Alessandro Rubini.  It is designed to account for system dependent
 * changes between the 2.0.36, 2.2.12, and 2.2.14 kernels that come in the 
 * RedHat 5.2, 6.1, and 6.2 distributions respectively.
 *
 *
 */


#ifndef LINUX_VERSION_CODE
#  include <linux/version.h>
#endif

#ifndef VERSION_CODE
#  define VERSION_CODE(vers,rel,seq) ( ((vers)<<16) | ((rel)<<8) | (seq) )
#endif


/* Check for supported kernel levels */

#if LINUX_VERSION_CODE < VERSION_CODE(2,0,0)
#  error "This kernel is too old: not supported by this file"
#
#elif LINUX_VERSION_CODE > VERSION_CODE(2,3,0)
#  error "This kernel is too new: not supported by this file"
#
#endif

#include <linux/types.h> /* used later in this header */



/* Modularization issues */

#if LINUX_VERSION_CODE < VERSION_CODE(2,1,18)
#  define EXPORT_NO_SYMBOLS register_symtab(NULL)
#  define __MODULE_STRING(s)         /* nothing */
#  define MODULE_PARM(v,t)           /* nothing */
#  define MODULE_PARM_DESC(v,t)      /* nothing */
#  define MODULE_AUTHOR(n)           /* nothing */
#  define MODULE_DESCRIPTION(d)      /* nothing */
#  define MODULE_SUPPORTED_DEVICE(n) /* nothing */
#endif



/* File operation proto type changes */

#if LINUX_VERSION_CODE < VERSION_CODE(2,1,0)
#  define count_t  int
#  define rw_t     int
#else
#  define count_t  unsigned long
#  define rw_t     long
#endif

#if LINUX_VERSION_CODE > VERSION_CODE(2,1,22)
#  include <linux/poll.h>
#else
#  define __USE_OLD_SELECT__
#endif

#if LINUX_VERSION_CODE < VERSION_CODE(2,1,31)
#  define close_t void
#  define close_return(x) return
#else
#  define close_t int
#  define close_return(x) return (x)
#endif




/* Access to user space */

#if LINUX_VERSION_CODE < VERSION_CODE(2,1,0)
#  include <asm/segment.h>
#  include <linux/mm.h>
#  define   access_ok(t,a,sz)          (verify_area((t),(a),(sz)) ? 0 : 1)
#  define   __copy_to_user(t,f,n)      (memcpy_tofs(t,f,n), 0)
#  define   __copy_from_user(t,f,n)    (memcpy_fromfs((t),(f),(n)), 0)
#  define   __PUT_USER(val,add)        (put_user((val),(add)), 0)
#  define   __GET_USER(dest,add)       ((dest)=get_user((add)), 0)
#else
#  include <asm/uaccess.h>
#  define   __PUT_USER                 __put_user
#  define   __GET_USER                 __get_user
#endif



/* Atomic changes from int to a structure int */

#if LINUX_VERSION_CODE < VERSION_CODE(2,1,0)
#  define ATOMIC_INIT(i)     (i)
#  define atomic_read(v)     (*v)
#  define atomic_set(v,i)    ((*v) = (i))
#endif


#if LINUX_VERSION_CODE < VERSION_CODE(2,1,30)
#  define in_interrupt() (intr_count!=0)
#endif




/* Module init/cleanup and dispatch function prototypes */

int     init_module(     void );
void    cleanup_module(  void );

int     Linux_Open(  struct inode *inode, struct file *filp );
close_t Linux_Close( struct inode *inode, struct file *filp );
int     Linux_Ioctl( struct inode *inode, struct file *filp,
                                     unsigned int cmd, unsigned long arg );
#if LINUX_VERSION_CODE < VERSION_CODE(2,2,0)
rw_t    Linux_Read(  struct inode *inode, struct file *filp,
                                                char *buf, count_t count );
#else
ssize_t Linux_Read(  struct file *filp, char *buf, size_t count, 
                                               loff_t *offset);
#endif




struct file_operations sr_fops = {
        open:    Linux_Open,
        release: Linux_Close,
        read:    Linux_Read,
        ioctl:   Linux_Ioctl,
        };



struct timer_list ktimer;





/* Protos from <linux/ioport.h> */

int Linux_check_region( unsigned long from, unsigned long extent ) {

        return( check_region( from, extent ) );
}

void Linux_request_region( unsigned long from, unsigned long extent, 
                                                        const char* name ) {

        request_region( from, extent, name );
}

void Linux_release_region( unsigned long from, unsigned long extent ) {

        release_region( from, extent );
}



/* Protos from <linux/fs.h> */

int Linux_register_chrdev( unsigned int major, const char* name ) {

        return( register_chrdev( major, name, &sr_fops ) ); 
}

int Linux_unregister_chrdev( unsigned int major, const char* name ) {

        return( unregister_chrdev( major, name ) );
}




/* Protos from <asm/uaccess.h> */

int Linux_access_ok( int type, const void* arg, unsigned long size ) {

        if (type == CHECK_WRITE)
                return( access_ok( VERIFY_WRITE, arg, size ) );

        else if (type == CHECK_READ)
                return( access_ok( VERIFY_READ, arg, size ) );

        else
                return( 1 );
}

void Linux_get_user_uchar( unsigned char *value, unsigned char* iobuffer ) {

        __GET_USER( *value, iobuffer );
}

void Linux_get_user_int( int *value, int* iobuffer ) {

        __GET_USER( *value, iobuffer );
}

void Linux_put_user_uchar( unsigned char value, unsigned char* iobuffer ) {

        __PUT_USER( value, iobuffer );
}

void Linux_put_user_int( int value, int* iobuffer ) {

        __PUT_USER( value, iobuffer );
}

unsigned long Linux_copy_from_user( void* tokernel, const void* fromuser,
                                                         unsigned long size ) {

        __copy_from_user( tokernel, fromuser, size);
	return( size );
}

unsigned long Linux_copy_to_user( void* touser, const void* fromkernel, 
                                                         unsigned long size) {

        __copy_to_user( touser, fromkernel, size );
	return( size );
}




/* Protos from <linux/sched.h> */

unsigned long Linux_jiffies( void ) {

        return( jiffies );
}


int Linux_request_irq( unsigned int irq,
		       void (*handler)(int, void *, struct pt_regs *),
		       unsigned long flags, const char *device, void *dev_id ){

        return( request_irq( irq, handler, flags, device,dev_id) );
}

void Linux_free_irq( unsigned int irq, void *dev_id ) {

        free_irq( irq, dev_id );
}



/* Protos from <linux/timer.h>, plus 2 additional helper functions */

void Linux_init_timer( void *pktimer, PKFN_TIMER function, unsigned long data){

        struct timer_list* timer;

        timer = (struct timer_list *)pktimer;


        init_timer( timer );

        timer->function = function;
        timer->data     = data;

}

void Linux_add_timer( void *pktimer ) {

        add_timer( (struct timer_list *)pktimer );
}

int Linux_del_timer( void *pktimer ) {

        return( del_timer( (struct timer_list *)pktimer ) );
}

void* Linux_alloc_timer( void ) {

        return( (void*) &ktimer );
}

void Linux_set_timer_expires( void *pktimer, unsigned long period ) {

        struct timer_list* timer;

        timer = (struct timer_list *)pktimer;

        timer->expires  = jiffies + period;
}





/* Other functions */

void Linux_export_no_symbols( void ) {

        EXPORT_NO_SYMBOLS;
}

void Linux_mod_inc_use_count( void ) {

        MOD_INC_USE_COUNT;
}

void Linux_mod_dec_use_count( void ) {

        MOD_DEC_USE_COUNT;
}









/****************************************************************************/

/* DSP32C specific variables and functions */



/* Allow these variables to be overridden from the insmod command line. */

int port_baseio  = -1;
int port_basesel = -1;
int port_count   = -1;
int dev_major    = -1;

MODULE_DESCRIPTION("Symmetric Research DSP32C Coprocessor Linux Device Driver");
MODULE_PARM( port_baseio,  "i" );
MODULE_PARM( port_basesel, "i" );
MODULE_PARM( port_count,   "i" );
MODULE_PARM( dev_major,    "i" );







/* Module init and cleanup functions called by kernel (insmod) */

int init_module( void ) {

        return( dsp32c_init_module() );
}
void cleanup_module(  void ) {

        dsp32c_cleanup_module( );
}



/* "Dispatch" functions called by kernel (open, read, ioctl) */

int Linux_Open(  struct inode *inode, struct file *filp ) {

        return( dsp32c_open( &filp->private_data ) );
}

close_t Linux_Close( struct inode *inode, struct file *filp ) {

        dsp32c_close( filp->private_data );

        close_return( 0 );
}

int Linux_Ioctl( struct inode *inode, struct file *filp, unsigned int cmd, 
                                                         unsigned long arg ) {

        return( dsp32c_ioctl( filp->private_data, cmd, arg ) );
}

#if LINUX_VERSION_CODE < VERSION_CODE(2,2,0)
rw_t    Linux_Read(struct inode *inode, struct file *filp,
                                                char *buf, count_t count ) {
#else
ssize_t Linux_Read(struct file *filp, char *buf, size_t count, loff_t *offset){
                                               
#endif

        return( 0 );
}


