/* FILE: stack.dsp              Copyright (c), Symmetric Research, 1992-1999
 *
 * This program demonstates using a software stack for nested subroutine
 * function calls.  Main calls transform which in turn calls scale and
 * add.
 * 
 * To run this program, assemble it with DSPASM, and then run it by hand
 * in DSPMON.
 *
 * 
 * NOTES:
 *
 * When the DSP32C makes a function call it places the return address
 * into a CAU register.  If a subseqent function call is made, then the
 * first address will be overwritten, and the thread of execution lost.
 *
 * The DSP32C does not automatically maintain a return address stack
 * for subroutine function calls, so it becomes the responsibility of
 * the programmer to do so.
 *
 * The way to take care of this is to maintain a software stack, and
 * save and restore the return address register on entry and exit to any
 * functions which will be making subseqent function calls.
 *
 * In the AT&T/Lucent assembler mnemonics the register to be used for
 * storing the return address can be specified in the call and return
 * instructions.  In DSPASM, register r14 is used by default, and is not
 * specified in the call and return instructions.
 *
 *
 */

constant

        MAIN            = 0x000000,
        DATA            = 0x000100,
        FUNCTIONS       = 0x000150;



<DATA>

dspf x[] = { 1.0, 2.0, 3.0, 4.0 }, A = 2.0, B = 1.0;






/* SUBROUTINE FUNCTIONS:
 *
 *
 *
 */

<FUNCTIONS>

scale {


        /* Scale each entry of x by A. */

        r1 = x;
        r2 = x;
        r3 = A;

        do 4
                { *r1++ = a0 = *r2++ * *r3; }


        /* Wait for DAU pipeline to flush, and return. */

        nop; nop; nop; return;
        

/* NOTES:
 *
 * Save and restore of the return address register r14 is not required
 * in the functions scale or add because they do not call any one else.
 *
 * Also, DSPASM does NOT automatically emit a return instruction at the
 * end of functions like C.  You *MUST* program the return, or else
 * execution will continue right off the end of the function.
 *
 * Finally, note that the functions are listed in Pascal like order to
 * avoid any forward references in the code.  DSPASM does not allow
 * forward references.
 *
 */


}
add {

        /* Add a constant B to each entry of x. */

        r1 = x;
        r2 = x;
        r3 = B;

        do 4
                { *r1++ = a0 = *r2++ + *r3; }


        /* Wait for DAU pipeline to flush, and return. */

        nop; nop; nop; return;


}
transform { /* scale and add a constant to each entry of x. */


        /* Save the return address, pushing r14. */

        *r13++ = r14;


        /* Make some function calls. */

        call scale;
        call add;


        /* Pop, restoring r14, and return. */

        r13 = r13 - 4; r14 = *r13; return;
}





/* SUBROUTINE STACK:
 *
 * Only a shallow stack is required here because the function calls are
 * not nested very deeply.
 *
 * With the programming above, the stack will grow towards HIGHER
 * addresses in memory.  Running stacks that grow towards LOWER addresses
 * is also fine.
 *
 * While this stack could be located anywhere in memory, putting it at
 * the end of the functions is reasonable because that way it is less
 * likely to overwrite any program areas as it grows.
 *
 *
 */

long stack[5];






/* MAIN:
 *
 * Set up the stack and make some subroutine calls.
 *
 *
 *
 *
 */

<MAIN>

main {

        /* Use r13 as a dedicated stack pointer. */

        r13 = stack;


        /* Call a function to transform x. */

        call transform;


        /* Wait for the PC. */

        wait: goto wait;
}
